package com.nisum.example.mongodb.service;

import com.nisum.example.mongodb.model.Employee;
import com.nisum.example.mongodb.repository.EmployeeRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

/**
 *Service class having methods for CURD operations
 * Methods having Mono<T> and Flux<T> as return types
 */
@Service
public class EmployeeServiceImpl implements IEmployeeService {

    @Autowired
    private EmployeeRepository employeeRepository;

    /**
     * perform create operation
     * @param emp
     * @return Mono
     */
    @Override
    public Mono<Employee> insert(Employee emp) {
        return employeeRepository.save(emp);
    }

    /**
     * perform fetch operation based on ID
     * @param id
     * @return Mono
     */
    @Override
    public Mono<Employee> fetchById(Integer id) {
        return employeeRepository.findById(id);

    }


    /**
     * perform fetch operation based on name
     * @param name
     * @return Flux
     */
    @Override
    public Flux<Employee> fetchByName(String name) {
        return employeeRepository.findByName(name);
    }

    /**
     * perform fetch all records
     * @return Flux
     */
    @Override
    public Flux<Employee> fetchAll() {
        return employeeRepository.findAll()
                /*.delayElements(Duration.ofSeconds(1)).map(
                        employee -> {
                            employee.setSalary(4000L);
                            System.out.println(employee);
                            return employee;
                        })*/;
    }

    /**
     * perform modify of object
     * @param emp
     * @return updated object of type Mono
     */
    @Override
    public Mono<Employee> modify(Employee emp) {
        return employeeRepository.save(emp);
    }

    /**
     * perform removeById based on id
     * @param id
     * @return Mono
     */
    @Override
    public Mono<Void> removeById(Integer id) {
        return employeeRepository.deleteById(id);
    }
}
